@@ -217,6 +217,29 @@ function startFarabi(editorId) {
 		});
 	});
 
+	$(".perl_execute_button").click(function() {
+		$.post('/perl_execute', {"source": editor.getValue()}, function(data) {
+			var $output = $('#output');
+			$output.val('');
+			
+			// Handle exit code
+			if (data.exit > 0) {
+				humane.log("Exit code: " + data.exit);
+			}
+			
+			// Handle STDERR
+			if (data.stderr.length) {
+				$output.val(data.stderr + "\n");
+			}
+			
+			// Handle STDOUT
+			$output.val($output.val() + data.stdout);
+			
+			// Show output tab
+			$('a[href="#output-tab"]').tab('show');
+		});
+	});
+
 	var perlCriticWidgets = [];
 
 	var perl_critic = function() {
@@ -357,6 +380,16 @@ function startFarabi(editorId) {
 		editor.setOption('lineNumbers', $(this).is(':checked'));
 	});
 
+	$("#pod_viewer_checkbox").change(function() {
+		if ( $(this).is(':checked') ) {
+			$("#pod-tab").addClass("span6").show();
+			$("#editor-border").removeClass("span12").addClass("span6");
+		} else {
+			$("#pod-tab").removeClass("span6").hide();
+			$("#editor-border").removeClass("span6").addClass("span12");
+		}
+	});
+
 	$("#tab_size").change(function() {
 		var tabSize = $(this).val();
 		if($.isNumeric(tabSize)) {
